(*********************************************************************)
(*                         Diy                                       *)
(*                                                                   *)
(*   Jade Alglave, Luc Maranget INRIA Paris-Rocquencourt, France.    *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique. All rights reserved. This file is distributed    *)
(*  under the terms of the Lesser GNU General Public License.        *)
(*********************************************************************)

open Printf
open Code

module Make(V:Constant.S) : XXXCompile.S =
struct

  module PPC = PPCArch.Make(V)
  include CompileCommon.Make(PPC) 
  open C

  let next_reg x = PPC.alloc_reg x

  let next_init st p init loc =
    let rec find_rec = function
      | (PPC.Reg (p0,r0),loc0)::_ when loc0 = loc && p = p0 ->
          r0,init,st
      | _::rem -> find_rec rem
      | [] ->
          let r,st = next_reg st in
          r,(PPC.Reg (p,r),loc)::init,st in
    find_rec init

  let pseudo = List.map (fun i -> PPC.Instruction i)

  let emit_load st p init x =
    let rA,st = next_reg st in
    let rB,init,st = next_init st p init x in
    rA,init,pseudo [PPC.Plwz (rA,0,rB)],st

  and emit_store st p init x v =
    let rA,st = next_reg st in
    let rB,init,st = next_init st p init x in
    init,pseudo [PPC.Pli (rA,v) ; PPC.Pstw (rA,0,rB)],st

  and emit_load_idx st p init x idx =
    let rA,st = next_reg st in
    let rB,init,st = next_init st p init x in
    rA,init,pseudo [PPC.Plwzx (rA,idx,rB)],st

  and emit_store_idx st p init x idx v =
    let rA,st = next_reg st in
    let rB,init,st = next_init st p init x in
    init,pseudo [PPC.Pli (rA,v) ; PPC.Pstwx (rA,idx,rB)],st

  let emit_access  st p init e = match e.dir with
  | R ->
      let r,init,cs,st = emit_load st p init e.loc in
      Some r,init,cs,st
  | W ->
      let init,cs,st = emit_store st p init e.loc e.v in
      None,init,cs,st

  let emit_access_dep st p init e r1 =
    let r2,st = next_reg st in
    let c = PPC.Pxor(PPC.DontSetCR0,r2,r1,r1) in
    match e.dir with
    | R ->
        let r,init,cs,st = emit_load_idx st p init e.loc r2 in
        Some r,init, PPC.Instruction c::cs,st
    | W ->
        let init,cs,st = emit_store_idx st p init e.loc r2 e.v in
        None,init,PPC.Instruction c::cs,st

  let emit_access_ctrl st p init e r1 =
    let lab = Label.next_label "LC" in
    let c =
      [PPC.Instruction (PPC.Pcmpw (0,r1,r1));
       PPC.Instruction (PPC.Pbcc (PPC.Eq,lab));
       PPC.Label (lab,PPC.Nop);] in
    match e.dir with
    | R ->
        let r,init,cs,st = emit_load st p init e.loc in
        Some r,init, c@[PPC.Instruction PPC.Pisync]@cs,st
    | W ->
        let init,cs,st = emit_store st p init e.loc e.v in
        None,init,c@cs,st


  let emit_fence f =
    PPC.Instruction
      (match f with
      | PPC.Sync -> PPC.Psync
      | PPC.LwSync -> PPC.Plwsync)

end
