(*********************************************************************)
(*                         Diy                                       *)
(*                                                                   *)
(*   Jade Alglave, Luc Maranget INRIA Paris-Rocquencourt, France.    *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique. All rights reserved. This file is distributed    *)
(*  under the terms of the Lesser GNU General Public License.        *)
(*********************************************************************)

open Printf
open Misc
open Code
let nprocs = ref 4
let size = ref 6
let one = ref false 
let arch = ref PPC
let tarfile = ref None
let safes = ref None 
let relaxs = ref None
let name = ref None
let cannonical_only = ref true
let conf = ref None
let mode = ref Sc 
let do_observers = ref false
let upto = ref true
let optcond = ref true
let overload = ref None
let switch = ref false

let parse_mode s =
  match s with 
    | "sc" -> Sc
    | "thin" -> Thin
    | "uni" -> Uni
    | _ -> failwith "Wrong mode, choose sc or thin or uni"

let common_specs =
  ("-v", Arg.Unit (fun () -> incr Misc.verbose),"  be verbose")::
  ("-version", Arg.Unit (fun () -> print_endline Version.version ; exit 0),
   " show version number and exit")::
  ("-obs", Arg.Set do_observers, " enable observers")::
  ("-optcond", Arg.Set optcond, " optimize conditions (default)")::
  ("-nooptcond", Arg.Clear optcond, "do not optimize conditions")::
  ("-overload", Arg.Int (fun n -> overload := Some n),
   "<n> stress load unit by <n> useless loads")::
  ("-switch", Arg.Set switch, " switch something")::
  []

let speclist =
  common_specs @
   ("-mode", Arg.String (fun s -> mode := parse_mode s), "<sc|thin|uni> running mode, experimental, default sc")::
   ("-conf", Arg.String (fun s -> conf := Some s), "<file> read configuration file")::
   ("-o", Arg.String (fun s -> tarfile := Some s),
    "<name.tar> output litmus tests in archive <name.tar> (default, output in curent directory)")::
   ("-size", Arg.Int (fun n -> size := n),
    sprintf
      "<n> set the maximal size of cycles (default %i)"
      !size)::
   ("-exact", Arg.Clear upto, " produce cycle of size exactly <n>")::
   ("-nprocs", Arg.Int (fun n -> nprocs := n),
   sprintf "<n> reject tests with more than <n> threads  (default %i)"
      !nprocs)::
   ("-c", Arg.Bool (fun b ->  cannonical_only := b),
   sprintf "<b> avoid equivalent cycles (default %b)" !cannonical_only)::
   ("-one", Arg.Unit (fun _ -> one := true), 
    "<relax-list> specify a sole cycle")::
   ("-relax", Arg.String (fun s -> relaxs := Some s),
    "<relax-list> specify a relax list")::
   ("-safe", Arg.String (fun s -> safes := Some s),
    "<relax-list> specify a safe list")::
   ("-name",Arg.String (fun s -> name := Some s),
     "<s> specify base name of tests")::
  Util.arch_opt (fun a -> arch := a) (pp_arch !arch)

let usage_msg = "Usage: " ^ Sys.argv.(0) ^   "[options]*"
