/*********************************************************************/
/*                          Litmus                                   */
/*                                                                   */
/*        Luc Maranget, INRIA Paris-Rocquencourt, France.            */
/*        Susmit Sarkar, University of Cambridge, UK.                */
/*                                                                   */
/*  Copyright 2010 Institut National de Recherche en Informatique et */
/*  en Automatique and the authors. All rights reserved.             */
/*  This file is distributed  under the terms of the Lesser GNU      */
/*  General Public License.                                          */
/*********************************************************************/

#include <pthread.h>
/********/
/* Misc */
/********/

void err(int e, char *msg) ;

void *malloc_check(size_t sz) ;


/************************/
/* Command line options */
/************************/

typedef struct {
  int verbose ;
  int max_run ;
  int size_of_test ;
  int n_exe ;
  char *run ;
  int sync_n ;
} cmd_t ;

void parse_cmd(int argc, char **argv, cmd_t *def, int n, cmd_t *p) ;


/********************/
/* Thread utilities */
/********************/

/* Mutex */

typedef pthread_mutex_t pm_t ;

pm_t *pm_create(void) ;
void pm_free(pm_t *p) ;
void pm_lock(pm_t *m) ;
void pm_unlock(pm_t *m) ;

/* Condition variable */

typedef struct {
  pm_t *c_mutex ;
  pthread_cond_t *c_cond ;
} pc_t ;

pc_t *pc_create(void) ;
void pc_free(pc_t *p) ;
void pc_wait(pc_t *p) ;
void pc_broadcast (pc_t *p) ;

/* Barrier */

typedef struct {
  pc_t *cond ;
  volatile int count ;
  int nprocs ; 
} pb_t ;


pb_t *pb_create(int nprocs) ;
void pb_free(pb_t *p) ;
void pb_wait(pb_t *p) ;


/* Or flag */

typedef struct {
  pc_t *cond ;
  int nprocs ;
  int count ;
  volatile int val ;
} po_t ;

po_t *po_create(int nprocs) ;
void po_free(po_t *p) ;
/* Initialize flag, must be called by all participant */
void po_reinit(po_t *p) ;
/* Return the 'or' of the v arguments of all participants */
int po_wait(po_t *p, int v) ;

/* Thread launch and join */

typedef void* f_t(void *);

void launch(pthread_t *th, f_t *f, void *a) ;

void *join(pthread_t *th) ;


/*****************/
/* Random things */
/*****************/

typedef unsigned st_t ;
/* Unlocked random bit */

int rand_bit(st_t *st) ;

/* permutations */

void perm_ints(st_t *st,int t[], int sz) ;
void perm_funs(st_t *st,f_t *t[], int sz) ;
void perm_threads(st_t *st,pthread_t t[], int sz) ;




/*********************/
/* Real time counter */
/*********************/

typedef unsigned long long tsc_t ;
#define PTSC "%llu"

/* Result in micro-seconds */
tsc_t timeofday(void) ;
double tsc_ratio(tsc_t t1, tsc_t t2) ;
double tsc_millions(tsc_t t) ;
