(*********************************************************************)
(*                          Litmus                                   *)
(*                                                                   *)
(*        Luc Maranget, INRIA Paris-Rocquencourt, France.            *)
(*        Susmit Sarkar, University of Cambridge, UK.                *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique and the authors. All rights reserved.             *)
(*  This file is distributed  under the terms of the Lesser GNU      *)
(*  General Public License.                                          *)
(*********************************************************************)

(******************************)
(* Test configuration control *)
(******************************)

val get_gcc_opts : Misc.arch -> string
val set_gcc_opts : string -> unit
val default_gcc_opts : Misc.arch -> string

(* Target OS *)
type target_os = Linux | Mac | AIX | AIX5

val get_target_os : Misc.arch -> target_os
val set_os : string -> unit
val default_os : unit -> string

(* 32/64 mode (impacts on code generation) *)
type word = W32 | W64

val get_word :  Misc.arch -> word
val set_word : string -> unit
val default_ws : unit -> string

(* A choice for barriers *)
type barrier = User | Pthread | NoBarrier
val default_barrier : unit -> string
val get_barrier : Misc.arch -> barrier
val set_barrier : string -> unit
val pp_barrier : Misc.arch -> string

(* how threads are started/joined *)
type launch = Fixed | Changing
val default_launch : unit -> string
val get_launch : Misc.arch -> launch
val set_launch : string -> unit
val pp_launch : Misc.arch -> string

(* how memory is accessed *)
type memory = Direct | Indirect
val default_memory : unit -> string
val get_memory : Misc.arch -> memory
val set_memory : string -> unit
val pp_memory : Misc.arch -> string

(* Safer mode that add a few checks by fear of
   pthread not doing the right thing
   (Have writes to memory 'flushed' before thread creation
    and thread exit.)
     - Check initial state transmission from main thread to
       children.
     - Put fake initial value in variables used to transmit
       register final states
     - Check stabilization  of shared variables.
*)
val set_safer : bool -> unit
val get_safer : Misc.arch -> bool
val default_safer : unit -> bool

(* Preload mode: Randomly load shared variabels before starting
   thread test code proper *)
val set_preload : bool -> unit
val get_preload : Misc.arch -> bool
val default_preload : unit -> bool

(* Flood avail procs *)
val is_flood : Misc.arch -> bool
val get_avail : Misc.arch -> int option
val set_avail : int -> unit

(* Flood technique *)
type para =
  | Shell (* use shell and Unix processes *)
  | Self  (* run several test instances ourselves [using threads] *)

val set_para : string -> unit
val get_para : unit -> para
val pp_para : unit -> string
val default_para : unit -> string

(* Limit number of procs *)
val get_limit : unit -> bool
val set_limit : bool -> unit

(* Set size and number of runs *)
val default_size : unit -> int
val default_run : unit -> int
val set_size : int -> unit
val set_run : int -> unit
val dump_bench : Misc.arch -> out_channel -> unit


(******************)
(* litmus control *)
(******************)

val get_tsc : unit -> bool
val set_tsc : bool -> unit

val get_clocktime : unit -> bool
val set_clocktime : bool -> unit

val get_profiling : unit -> bool
val set_profiling : bool -> unit

(* Sync macro replacement *)
val sync_const : int (* One place for magic constant 128 *)

val get_sync_macro : unit -> int option
val set_sync_macro : int -> unit

(* Stop test as soon as condition is settled *)
val get_speedcheck : unit -> bool
val set_speedcheck : bool -> unit
val default_speedcheck : unit -> bool

(* Run name - identifying string *)
val get_runname : unit -> string
val set_runname : string -> unit

(* Cross compilation *)
val is_cross : unit -> bool
val set_cross : string -> unit
val get_cross : unit -> string


