(*********************************************************************)
(*                          Litmus                                   *)
(*                                                                   *)
(*        Luc Maranget, INRIA Paris-Rocquencourt, France.            *)
(*        Susmit Sarkar, University of Cambridge, UK.                *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique and the authors. All rights reserved.             *)
(*  This file is distributed  under the terms of the Lesser GNU      *)
(*  General Public License.                                          *)
(*********************************************************************)

{
open Globals

exception Error of string

let set_int set arg =
  let x =
    try int_of_string arg
    with Failure _ ->  raise (Error "int parameter expected") in
  set x

let set_bool set arg =
  let x =
    try bool_of_string arg
    with Failure _ ->  raise (Error "bool parameter expected") in
  set x


}

let blank = [' ''\t''\r']
let not_blank = [^' ''\t''\n''\r']
let arg = (blank* '='? blank* (not_blank* as arg) blank* '\n')

rule main = parse
| eof
    { () }
| '#' [^'\n']* '\n'
| blank* '\n'
    { main lexbuf }
| ("gcc_opts"|"ccopts") arg 
    { set_gcc_opts arg ; main lexbuf }
| "barrier" arg
    { set_barrier arg ; main lexbuf }
| "launch" arg
    { set_launch arg ; main lexbuf }
| "size_of_test" arg
    { set_int set_size arg ; main lexbuf }
| "limit" arg 
    { set_bool set_limit arg ; main lexbuf }
| "number_of_run" arg
    { set_int set_run arg ; main lexbuf }
| "memory" arg
   { set_memory arg ; main lexbuf }
| "safer" arg
   { set_bool set_safer arg ; main lexbuf }
| "preload" arg
   { set_bool set_preload arg ; main lexbuf }
| "os" arg
   { set_os arg ; main lexbuf }
| ("word"|"ws") arg
  { set_word arg ; main lexbuf }
| "avail" arg
  { set_int set_avail arg ; main lexbuf }
| [^'\n']* as lxm '\n'?
  { raise (Error (Printf.sprintf "format: %s" lxm)) }
{

let lex chan = main (Lexing.from_channel chan)

}
